#include <windows.h>
#include <d3d9.h>
#include <d3dx9.h>
#include <fstream>
#include <vector>
#include "stage.h"
// Xe[W\t@C̓ǂݍ
StateInfo Stage::State_LoadingStageConstructFile::run(Stage *parent, State<Stage>* &newState) {
	// Xe[W\t@CI[v
	std::ifstream ifs(fileName.c_str());
	if (!ifs.is_open()) {
		newState = new State_OpenError;
		return StateInfo_Immediate;
	}
	std::vector<Variants> list = StrUtil::parse(ifs);
	newState = new State_ConstructStage(list);
	return StateInfo_Immediate;
}

// t@CI[vG[
StateInfo Stage::State_OpenError::run(Stage *parent, State<Stage>* &newState) {
	::MessageBoxA(NULL, "t@CI[vɎs܂", "", 0);
	newState = new State_Idle;
	return StateInfo_Immediate;
}

// Xe[W\[X쐬
StateInfo Stage::State_ConstructStage::run(Stage *parent, State<Stage>* &newState) {
	for (unsigned i = 0; i < ary.size(); i++) {
		Variants &v = ary[i];
		if (v[0].str == "Object") {
			// IuWFNg쐬
			int x = v[2].iVal;
			int y = v[3].iVal;
			ID3DXMesh *mesh = 0;
			if (v[1].str == "Sphere") {
				float radius = (float)v[4].iVal;
				D3DXCreateSphere(parent->dev, radius, 10, 10, &mesh, 0);
			} else if (v[1].str == "Box") {
				float w = (float)v[4].iVal;
				float h = (float)v[5].iVal;
				float d = (float)v[6].iVal;
				D3DXCreateBox(parent->dev, w, h, d, &mesh, 0);
			}
			parent->objects.push_back(new ObjectInfo(x, y, mesh));
		}
		else if (v[0].str == "StageName") {
			parent->stageName = v[1].str;
		}
	}
	newState = new State_FadeIn;
	return StateInfo_Immediate;
}

// tF[h
StateInfo Stage::State_FadeIn::run(Stage *parent, State<Stage>* &newState) {
	parent->fader.trans(curVal);
	if (curVal == 0) {
		newState = new State_StartGame(parent->dev);
		return StateInfo_Immediate;
	}
	const unsigned def = 4;
	curVal = (curVal <= def) ? 0 : curVal - def;
	return StateInfo_Next;
}

// Q[Jn}
Stage::State_StartGame::State_StartGame(IDirect3DDevice9 *dev) : brink(), counter(), bBrink(true) {
	D3DXCreateFontA(dev, 60, 30, 1000, 1, FALSE, SHIFTJIS_CHARSET, OUT_DEFAULT_PRECIS, ANTIALIASED_QUALITY, FIXED_PITCH | FF_DONTCARE, "lr SVbN", &font);
}

Stage::State_StartGame::State_StartGame() {
	font->Release();
}

StateInfo Stage::State_StartGame::run(Stage *parent, State<Stage>* &newState) {
	// Game Start ! _
	brink++;
	if (brink > 10)
		bBrink = false;
	if (brink >= 20) {
		bBrink = true;
		brink = 0;
		counter++;
		if (counter >= 6) {
			newState = new State_Gaming;
			return StateInfo_Next;
		}
	}
	return StateInfo_Continue;
}

void Stage::State_StartGame::draw(Stage *parent) {
	// Q[X^[g\
	if (bBrink) {
		RECT r = {0, 0, 0, 0}, r2 = {0, 0, 0, 0};
		font->DrawTextA(0, "Game Start !", -1, &r, DT_LEFT | DT_CALCRECT, 0xffffffff);
		::OffsetRect(&r, (640 - r.right) / 2, (480 - r.bottom) / 2);
		font->DrawTextA(0, "Game Start !", -1, &r, DT_LEFT, 0xffffffff);

		font->DrawTextA(0, parent->stageName.c_str(), -1, &r2, DT_LEFT | DT_CALCRECT, 0xffffffff);
		::OffsetRect(&r2, (640 - r2.right) / 2, (480 - r2.bottom) / 2 - 60);
		font->DrawTextA(0, parent->stageName.c_str(), -1, &r2, DT_LEFT, 0xffffffff);
	}
}

// Q[
StateInfo Stage::State_Gaming::run(Stage *parent, State<Stage>* &newState) {
	return StateInfo_Continue;
}

// ACh
StateInfo Stage::State_Idle::run(Stage *parent, State<Stage>* &newState) {
	return StateInfo_Continue;
}

Stage::Stage(IDirect3DDevice9 *dev, const char* stageFileName) : dev(dev), curState(new State_LoadingStageConstructFile(stageFileName)) {
}

Stage::~Stage() {
	for (unsigned i = 0; i < objects.size(); i++)
		delete objects[i];
	delete curState;
}

// XV
void Stage::update() {
	while(1) {
		State<Stage> *newState = 0;
		StateInfo info = curState->run(this, newState);
		if (newState) {
			delete curState;
			curState = newState;
		}
		if (info != StateInfo_Immediate)
			break;
	}
}

// `
void Stage::draw() {
	D3DXMATRIX view, proj;
	D3DXMatrixLookAtLH(&view, &D3DXVECTOR3(0.0f, -300.0f, -250.0f), &D3DXVECTOR3(0.0f, 0.0f, 0.0f), &D3DXVECTOR3(0.0f, 1.0f, 0.0f));
	D3DXMatrixPerspectiveFovLH(&proj, D3DXToRadian(30.0f), 1.33333f, 1.0f, 1000.0f);
	dev->SetTransform(D3DTS_VIEW, &view);
	dev->SetTransform(D3DTS_PROJECTION, &proj);
	// Cg
	D3DLIGHT9 light;
	ZeroMemory(&light, sizeof(D3DLIGHT9) );
	light.Direction = D3DXVECTOR3(1.0f, -1.0f, 1.0f);
	light.Type = D3DLIGHT_DIRECTIONAL; 
	light.Diffuse.r = 1.0f;
	light.Diffuse.g = 1.0f;
	light.Diffuse.b = 1.0f;
	light.Ambient.r = 0.5f;
	light.Ambient.g = 0.5f;
	light.Ambient.b = 0.5f;
	light.Range = 1000;
	dev->SetLight( 0, &light );
	dev->LightEnable( 0, true );
	dev->SetRenderState( D3DRS_LIGHTING, TRUE );
	dev->SetRenderState( D3DRS_AMBIENT, 0x00808080 );

	curState->draw(this);
	for (unsigned i = 0; i < objects.size(); i++)
		objects[i]->draw(dev);
	fader.draw(dev);
}
